@@ -42,7 +42,7 @@
 
 /**
  * SECTION:gsettings
- * @short_description: a high-level API for application settings
+ * @short_description: High-level API for application settings
  *
  * The #GSettings class provides a convenient API for storing and retrieving
  * application settings.
@@ -85,7 +85,16 @@
  * convention for schema ids is to use a dotted name, similar in
  * style to a DBus bus name, e.g. "org.gnome.font-rendering".
  *
- * <example><title>Default values</title>
+ * In addition to #GVariant types, keys can have types that have enumerated
+ * types. These can be described by a <tag class="starttag">choice</tag>,
+ * <tag class="starttag">enum</tag> or <tag class="starttag">flags</tag> element, see
+ * <xref linkend="schema-enumerated"/>. The underlying type of
+ * such a key is string, but you can use g_settings_get_enum(),
+ * g_settings_set_enum(), g_settings_get_flags(), g_settings_set_flags()
+ * access the numeric values corresponding to the string value of enum
+ * and flags keys.
+ *
+ * <example id="schema-default-values"><title>Default values</title>
  * <programlisting><![CDATA[
  * <schemalist>
  *   <schema id="org.gtk.test" path="/tests/" gettext-domain="test">
@@ -106,7 +115,7 @@
  * </schemalist>
  * ]]></programlisting></example>
  *
- * <example><title>Ranges, choices and enumerated types</title>
+ * <example id="schema-enumerated"><title>Ranges, choices and enumerated types</title>
  * <programlisting><![CDATA[
  * <schemalist>
  *
@@ -115,6 +124,12 @@
  *     <value nick="second" value="2"/>
  *   </enum>
  *
+ *   <enum id="myflags">
+ *     <value nick="flag1" value="1"/>
+ *     <value nick="flag2" value="2"/>
+ *     <value nick="flag3" value="4"/>
+ *   </enum>
+ *
  *   <schema id="org.gtk.test">
  *
  *     <key name="key-with-range" type="i">
@@ -139,6 +154,9 @@
  *       <default>'first'</default>
  *     </key>
  *
+ *     <key name='flags-key' flags='myflags'>
+ *       <default>["flag1",flag2"]</default>
+ *     </key>
  *   </schema>
  * </schemalist>
  * ]]></programlisting></example>
@@ -204,6 +222,7 @@ enum
   PROP_BACKEND,
   PROP_PATH,
   PROP_HAS_UNAPPLIED,
+  PROP_DELAY_APPLY
 };
 
 enum
@@ -417,6 +436,10 @@ g_settings_get_property (GObject    *object,
       g_value_set_string (value, settings->priv->schema_name);
       break;
 
+     case PROP_BACKEND:
+      g_value_set_object (value, settings->priv->backend);
+      break;
+
      case PROP_PATH:
       g_value_set_string (value, settings->priv->path);
       break;
@@ -425,6 +448,10 @@ g_settings_get_property (GObject    *object,
       g_value_set_boolean (value, g_settings_get_has_unapplied (settings));
       break;
 
+     case PROP_DELAY_APPLY:
+      g_value_set_boolean (value, settings->priv->delayed != NULL);
+      break;
+
      default:
       g_assert_not_reached ();
     }
@@ -674,6 +701,20 @@ g_settings_class_init (GSettingsClass *class)
                            FALSE,
                            G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
+   /**
+    * GSettings:delay-apply:
+    *
+    * Whether the #GSettings object is in 'delay-apply' mode. See
+    * g_settings_delay() for details.
+    *
+    * Since: 2.28
+    */
+   g_object_class_install_property (object_class, PROP_DELAY_APPLY,
+     g_param_spec_boolean ("delay-apply",
+                           P_("Delay-apply mode"),
+                           P_("Whether this settings object is in 'delay-apply' mode"),
+                           FALSE,
+                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 }
 
 /* Construction (new, new_with_path, etc.) {{{1 */
@@ -1458,8 +1499,28 @@ g_settings_set_value (GSettings   *settings,
   g_return_val_if_fail (key != NULL, FALSE);
 
   g_settings_get_key_info (&info, settings, key);
-  g_return_val_if_fail (g_settings_type_check (&info, value), FALSE);
-  g_return_val_if_fail (g_settings_key_info_range_check (&info, value), FALSE);
+
+  if (!g_settings_type_check (&info, value))
+    {
+      g_critical ("g_settings_set_value: key '%s' in '%s' expects type '%s', but a GVariant of type '%s' was given",
+                  key,
+                  settings->priv->schema_name,
+                  g_variant_type_peek_string (info.type),
+                  g_variant_get_type_string (value));
+
+        return FALSE;
+      }
+
+  if (!g_settings_key_info_range_check (&info, value))
+    {
+      g_warning ("g_settings_set_value: value for key '%s' in schema '%s' "
+                 "is outside of valid range",
+                 key,
+                 settings->priv->schema_name);
+
+        return FALSE;
+    }
+
   g_settings_free_key_info (&info);
 
   return g_settings_write_to_backend (&info, value);
@@ -1929,6 +1990,8 @@ g_settings_delay (GSettings *settings)
   g_settings_backend_watch (settings->priv->backend,
                             &listener_vtable, G_OBJECT (settings),
                             settings->priv->main_context);
+
+  g_object_notify (G_OBJECT (settings), "delay-apply");
 }
 
 /**
@@ -2159,7 +2222,7 @@ g_settings_list_keys (GSettings *settings)
  * time and you should connect to the "children-changed" signal to watch
  * for those changes.  Note that there is a race condition here: you may
  * request a child after listing it only for it to have been destroyed
- * in the meantime.  For this reason, g_settings_get_chuld() may return
+ * in the meantime.  For this reason, g_settings_get_child() may return
  * %NULL even for a child that was listed by this function.
  *
  * For GSettings objects that are not lists, you should probably not be
@@ -2511,7 +2574,7 @@ g_settings_bind_invert_boolean_set_mapping (const GValue       *value,
  * g_settings_bind:
  * @settings: a #GSettings object
  * @key: the key to bind
- * @object: a #GObject
+ * @object: (type GObject.Object): a #GObject
  * @property: the name of the property to bind
  * @flags: flags for the binding
  *
@@ -2565,7 +2628,7 @@ g_settings_bind (GSettings          *settings,
  * g_settings_bind_with_mapping:
  * @settings: a #GSettings object
  * @key: the key to bind
- * @object: a #GObject
+ * @object: (type GObject.Object): a #GObject
  * @property: the name of the property to bind
  * @flags: flags for the binding
  * @get_mapping: a function that gets called to convert values
@@ -2778,7 +2841,7 @@ g_settings_binding_writable_changed (GSettings   *settings,
  * g_settings_bind_writable:
  * @settings: a #GSettings object
  * @key: the key to bind
- * @object: a #GObject
+ * @object: (type GObject.Object):a #GObject
  * @property: the name of a boolean property to bind
  * @inverted: whether to 'invert' the value
  *
